@@ -18,6 +18,7 @@ from accessible import BoundingBox
 from exceptions import *
 
 from busutils import AccessibilityBus
+from factory import AccessibleFactory
 
 __all__ = [
                 "Event",
@@ -159,9 +160,9 @@ def event_type_to_signal_reciever(bus, factory, event_handler, event_type):
         if event_type.minor:
                 kwargs['arg0'] = event_type.minor
 
-        def handler_wrapper(minor, detail1, detail2, any_data,
+        def handler_wrapper(app, minor, detail1, detail2, any_data,
                             sender=None, interface=None, member=None, path=None):
-                event = Event((minor, detail1, detail2, any_data), factory, path, sender, interface, member)
+                event = Event((minor, detail1, detail2, any_data), factory, path, app, interface, member)
                 return event_handler(event)
 
         return bus.add_signal_receiver(handler_wrapper, **kwargs)
@@ -202,7 +203,7 @@ class Event(object):
         @ivar any_data: Extra AT-SPI data payload
         @type any_data: object
         @ivar host_application: Application owning the event source
-        @type host_application: Accessibility.Application
+        @type host_application: Tuple (Name, Path)
         @ivar source_name: Name of the event source at the time of event dispatch
         @type source_name: string
         @ivar source_role: Role of the event source at the time of event dispatch
@@ -256,18 +257,25 @@ class Event(object):
 
                 	data = event[3]
 
-                	if name == "object_bounds_changed":
+                	if self.type.is_subtype (EventType ("object:bounds-changed")):
                         	self.any_data = BoundingBox(*data)
+                        elif self.type.is_subtype (EventType ("object:children-changed")):
+				name, path = data;
+				self.any_data = self._acc_factory (name, path, interfaces.ATSPI_ACCESSIBLE)
+				self.any_data = ""
+			elif self.type.is_subtype (EventType ("object:property-change:parent")):
+				name, path = data;
+				self.any_data = self._acc_factory (name, path, interfaces.ATSPI_ACCESSIBLE)
                 	else:
                         	self.any_data = data
 
         @property
         def host_application(self):
-                #TODO TODO
                 if not self._application:
                         try:
-                                return self._acc_factory.create_application(self._source_application)
-                        except AccessibleObjectNotAvailable:
+                                name, path = self._source_application
+                                return self._acc_factory (name, path, interfaces.ATSPI_APPLICATION)
+                        except AccessibleObjectNoLongerExists:
                                 pass
                 return self._application
 
@@ -275,9 +283,9 @@ class Event(object):
         def source(self):
                 if not self._source:
                         try:
-                                self._source = self._acc_factory.create_accessible(self._source_application,
-                                                                                   self._source_path,
-                                                                                   interfaces.ATSPI_ACCESSIBLE)
+                                self._source = self._acc_factory (self._source_application,
+                                                                  self._source_path,
+                                                                  interfaces.ATSPI_ACCESSIBLE)
                         except AccessibleObjectNoLongerExists:
                                 pass
                 return self._source
@@ -308,19 +316,10 @@ class _ApplicationEventRegister (object):
 
         def __init__ (self):
                 self._bus = AccessibilityBus ()
-                self._cache = None
-                self._factory = None
+		self._factory = AccessibleFactory ()
+
                 self._event_listeners = {}
 
-                # All of this special casing is for the 'faked'
-                # events caused by cache updates.
-
-                self._name_type = EventType("object:property-change:name")
-                self._name_listeners = {}
-                self._description_type = EventType("object:property-change:description")
-                self._description_listeners = {}
-                self._parent_type = EventType("object:property-change:parent")
-                self._parent_listeners = {}
                 self._children_changed_type = EventType("object:children-changed")
                 self._children_changed_listeners = {}
 
@@ -328,54 +327,18 @@ class _ApplicationEventRegister (object):
                 for client in register.keys():
                         client(event)
 
-        def setCache (self, cache):
-                self._cache = cache
-
-        def setFactory (self, factory):
-                self._factory = factory
-
-        def notifyNameChange(self, name, path, acc_name):
-                event = Event(("accessible-name", 0, 0, acc_name),
-			      self._factory,
-                              path,
-                              name,
-                              "org.freedesktop.atspi.Event.Object",
-                              "property-change")
-                              
-                self._callClients(self._name_listeners, event)
-
-        def notifyDescriptionChange(self, name, path, acc_desc):
-                event = Event(("accessible-description", 0, 0, acc_desc),
-			      self._factory,
-                              path,
-                              name,
-                              "org.freedesktop.atspi.Event.Object",
-                              "property-change")
-                              
-                self._callClients(self._description_listeners, event)
-
-        def notifyParentChange(self, name, path):
-                event = Event(("accessible-parent", 0, 0, ""),
-			      self._factory,
-                              path,
-                              name,
-                              "org.freedesktop.atspi.Event.Object",
-                              "property-change")
-                              
-                self._callClients(self._parent_listeners, event)
-
-        def notifyChildrenChange(self, name, path, added):
+        def notifyChildrenChange(self, name, path, childref, added):
                 if added:
                         detail = "add"
                 else:
                         detail = "remove"
-                event = Event((detail, 0, 0, ""),
-			      self._factory,
+                event = Event((detail, 0, 0, childref),
+                              self._factory,
                               path,
                               name,
                               "org.freedesktop.atspi.Event.Object",
                               "children-changed")
-                              
+
                 self._callClients(self._children_changed_listeners, event)
 
         def _registerFake(self, type, register, client, *names):
@@ -421,7 +384,7 @@ class _ApplicationEventRegister (object):
                 if registered == []:
                         del(register[client])
 
-        def registerEventListener(self, client, *names):
+        def registerEventListener (self, client, *names):
                 try:
                         registered = self._event_listeners[client]
                 except KeyError:
@@ -433,9 +396,6 @@ class _ApplicationEventRegister (object):
                         registered.append((new_type.name,
                                            event_type_to_signal_reciever(self._bus, self._factory, client, new_type)))
 
-                self._registerFake(self._name_type, self._name_listeners, client, *names)
-                self._registerFake(self._description_type, self._description_listeners, client, *names)
-                self._registerFake(self._parent_type, self._parent_listeners, client, *names)
                 self._registerFake(self._children_changed_type, self._children_changed_listeners, client, *names)
 
         def deregisterEventListener(self, client, *names):
@@ -464,10 +424,6 @@ class _ApplicationEventRegister (object):
                 if registered == []:
                         del(self._event_listeners[client])
 
-                #TODO Do these account for missing also?
-                self._deregisterFake(self._name_type, self._name_listeners, client, *names)
-                self._deregisterFake(self._description_type, self._description_listeners, client, *names)
-                self._deregisterFake(self._parent_type, self._parent_listeners, client, *names)
                 self._deregisterFake(self._children_changed_type, self._children_changed_listeners, client, *names)
 
                 return missing
@@ -476,23 +432,8 @@ class _ApplicationEventRegister (object):
 
 class _NullApplicationEventRegister (object):
 
-        def setCache (self, cache):
-                pass
-
-        def setFactory (self, factory):
-                pass
-
-        def notifyNameChange(self, name, path, acc_name):
-                pass
-
-        def notifyDescriptionChange(self, name, path, acc_desc):
-                pass
-
-        def notifyParentChange(self, name, path):
-                pass
-
         def notifyChildrenChange(self, name, path, added):
-                pass
+		pass
 
         def registerEventListener(self, client, *names):
                 pass
